from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
  from ..models.extension_parameter_definitions import ExtensionParameterDefinitions





T = TypeVar("T", bound="ExtensionDetail")


@_attrs_define
class ExtensionDetail:
    """ 
        Attributes:
            id (str):
            version (str):
            parameter_definitions (List['ExtensionParameterDefinitions']):
     """

    id: str
    version: str
    parameter_definitions: List['ExtensionParameterDefinitions']


    def to_dict(self) -> Dict[str, Any]:
        from ..models.extension_parameter_definitions import (
            ExtensionParameterDefinitions,
        )
        id = self.id

        version = self.version

        parameter_definitions = []
        for parameter_definitions_item_data in self.parameter_definitions:
            parameter_definitions_item = parameter_definitions_item_data.to_dict()
            parameter_definitions.append(parameter_definitions_item)






        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "id": id,
            "version": version,
            "parameterDefinitions": parameter_definitions,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.extension_parameter_definitions import (
            ExtensionParameterDefinitions,
        )
        d = src_dict.copy()
        id = d.pop("id")

        version = d.pop("version")

        parameter_definitions = []
        _parameter_definitions = d.pop("parameterDefinitions")
        for parameter_definitions_item_data in (_parameter_definitions):
            parameter_definitions_item = ExtensionParameterDefinitions.from_dict(parameter_definitions_item_data)



            parameter_definitions.append(parameter_definitions_item)


        extension_detail = cls(
            id=id,
            version=version,
            parameter_definitions=parameter_definitions,
        )

        return extension_detail




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ClusterSettings")


@_attrs_define
class ClusterSettings:
    """ 
        Attributes:
            offload_enabled (bool):
            offload_timeout_min (int):
     """

    offload_enabled: bool
    offload_timeout_min: int


    def to_dict(self) -> Dict[str, Any]:
        offload_enabled = self.offload_enabled

        offload_timeout_min = self.offload_timeout_min


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "offloadEnabled": offload_enabled,
            "offloadTimeoutMin": offload_timeout_min,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        offload_enabled = d.pop("offloadEnabled")

        offload_timeout_min = d.pop("offloadTimeoutMin")

        cluster_settings = cls(
            offload_enabled=offload_enabled,
            offload_timeout_min=offload_timeout_min,
        )

        return cluster_settings




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
  from ..models.usage_cluster import UsageCluster





T = TypeVar("T", bound="UsageAdditionalPropertyItem")


@_attrs_define
class UsageAdditionalPropertyItem:
    """ 
        Attributes:
            id (str):
            name (str):
            clusters (List['UsageCluster']):
            used_storage (Union[Unset, float]):
     """

    id: str
    name: str
    clusters: List['UsageCluster']
    used_storage: Union[Unset, float] = UNSET


    def to_dict(self) -> Dict[str, Any]:
        from ..models.usage_cluster import UsageCluster
        id = self.id

        name = self.name

        clusters = []
        for clusters_item_data in self.clusters:
            clusters_item = clusters_item_data.to_dict()
            clusters.append(clusters_item)





        used_storage = self.used_storage


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "id": id,
            "name": name,
            "clusters": clusters,
        })
        if used_storage is not UNSET:
            field_dict["usedStorage"] = used_storage

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.usage_cluster import UsageCluster
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        clusters = []
        _clusters = d.pop("clusters")
        for clusters_item_data in (_clusters):
            clusters_item = UsageCluster.from_dict(clusters_item_data)



            clusters.append(clusters_item)


        used_storage = d.pop("usedStorage", UNSET)

        usage_additional_property_item = cls(
            id=id,
            name=name,
            clusters=clusters,
            used_storage=used_storage,
        )

        return usage_additional_property_item




import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.status import Status
from ..types import (
    UNSET,
    Unset,
)

if TYPE_CHECKING:
  from ..models.database_clusters import DatabaseClusters
  from ..models.database_integrations_item import DatabaseIntegrationsItem





T = TypeVar("T", bound="Database")


@_attrs_define
class Database:
    """ 
        Attributes:
            status (Status):
            id (str):
            name (str):
            clusters (DatabaseClusters):
            provider (str):
            region (str):
            created_at (datetime.datetime):
            created_by (str):
            integrations (Union[Unset, List['DatabaseIntegrationsItem']]):
            deleted_by (Union[Unset, str]):
            deleted_at (Union[Unset, datetime.datetime]):
     """

    status: Status
    id: str
    name: str
    clusters: 'DatabaseClusters'
    provider: str
    region: str
    created_at: datetime.datetime
    created_by: str
    integrations: Union[Unset, List['DatabaseIntegrationsItem']] = UNSET
    deleted_by: Union[Unset, str] = UNSET
    deleted_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)


    def to_dict(self) -> Dict[str, Any]:
        from ..models.database_clusters import DatabaseClusters
        from ..models.database_integrations_item import DatabaseIntegrationsItem
        status = self.status.value

        id = self.id

        name = self.name

        clusters = self.clusters.to_dict()

        provider = self.provider

        region = self.region

        created_at = self.created_at.isoformat()

        created_by = self.created_by

        integrations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.integrations, Unset):
            integrations = []
            for integrations_item_data in self.integrations:
                integrations_item = integrations_item_data.to_dict()
                integrations.append(integrations_item)





        deleted_by = self.deleted_by

        deleted_at: Union[Unset, str] = UNSET
        if not isinstance(self.deleted_at, Unset):
            deleted_at = self.deleted_at.isoformat()


        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
            "id": id,
            "name": name,
            "clusters": clusters,
            "provider": provider,
            "region": region,
            "createdAt": created_at,
            "createdBy": created_by,
        })
        if integrations is not UNSET:
            field_dict["integrations"] = integrations
        if deleted_by is not UNSET:
            field_dict["deletedBy"] = deleted_by
        if deleted_at is not UNSET:
            field_dict["deletedAt"] = deleted_at

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.database_clusters import DatabaseClusters
        from ..models.database_integrations_item import DatabaseIntegrationsItem
        d = src_dict.copy()
        status = Status(d.pop("status"))




        id = d.pop("id")

        name = d.pop("name")

        clusters = DatabaseClusters.from_dict(d.pop("clusters"))




        provider = d.pop("provider")

        region = d.pop("region")

        created_at = isoparse(d.pop("createdAt"))




        created_by = d.pop("createdBy")

        integrations = []
        _integrations = d.pop("integrations", UNSET)
        for integrations_item_data in (_integrations or []):
            integrations_item = DatabaseIntegrationsItem.from_dict(integrations_item_data)



            integrations.append(integrations_item)


        deleted_by = d.pop("deletedBy", UNSET)

        _deleted_at = d.pop("deletedAt", UNSET)
        deleted_at: Union[Unset, datetime.datetime]
        if isinstance(_deleted_at,  Unset):
            deleted_at = UNSET
        else:
            deleted_at = isoparse(_deleted_at)




        database = cls(
            status=status,
            id=id,
            name=name,
            clusters=clusters,
            provider=provider,
            region=region,
            created_at=created_at,
            created_by=created_by,
            integrations=integrations,
            deleted_by=deleted_by,
            deleted_at=deleted_at,
        )

        database.additional_properties = d
        return database

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ClusterSettingsUpdate")


@_attrs_define
class ClusterSettingsUpdate:
    """ 
        Attributes:
            offload_enabled (Union[Unset, bool]):
            offload_timeout_min (Union[Unset, int]):
     """

    offload_enabled: Union[Unset, bool] = UNSET
    offload_timeout_min: Union[Unset, int] = UNSET


    def to_dict(self) -> Dict[str, Any]:
        offload_enabled = self.offload_enabled

        offload_timeout_min = self.offload_timeout_min


        field_dict: Dict[str, Any] = {}
        field_dict.update({
        })
        if offload_enabled is not UNSET:
            field_dict["offloadEnabled"] = offload_enabled
        if offload_timeout_min is not UNSET:
            field_dict["offloadTimeoutMin"] = offload_timeout_min

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        offload_enabled = d.pop("offloadEnabled", UNSET)

        offload_timeout_min = d.pop("offloadTimeoutMin", UNSET)

        cluster_settings_update = cls(
            offload_enabled=offload_enabled,
            offload_timeout_min=offload_timeout_min,
        )

        return cluster_settings_update




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="Platform")


@_attrs_define
class Platform:
    """ 
        Attributes:
            id (str):
            name (str):
     """

    id: str
    name: str


    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "id": id,
            "name": name,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        platform = cls(
            id=id,
            name=name,
        )

        return platform




from http import HTTPStatus
from typing import (
    Any,
    Dict,
    List,
    Optional,
    Union,
    cast,
)

import httpx

from ... import errors
from ...client import (
    AuthenticatedClient,
    Client,
)
from ...models.database import Database
from ...types import (
    UNSET,
    Response,
)


def _get_kwargs(
    account_id: str,
    database_id: str,

) -> Dict[str, Any]:
    

    

    

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/accounts/{account_id}/databases/{database_id}".format(account_id=account_id,database_id=database_id,),
    }


    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Database]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Database.from_dict(response.json())



        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Database]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    database_id: str,
    *,
    client: AuthenticatedClient,

) -> Response[Database]:
    """ 
    Args:
        account_id (str):
        database_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Database]
     """


    kwargs = _get_kwargs(
        account_id=account_id,
database_id=database_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    account_id: str,
    database_id: str,
    *,
    client: AuthenticatedClient,

) -> Optional[Database]:
    """ 
    Args:
        account_id (str):
        database_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Database
     """


    return sync_detailed(
        account_id=account_id,
database_id=database_id,
client=client,

    ).parsed

async def asyncio_detailed(
    account_id: str,
    database_id: str,
    *,
    client: AuthenticatedClient,

) -> Response[Database]:
    """ 
    Args:
        account_id (str):
        database_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Database]
     """


    kwargs = _get_kwargs(
        account_id=account_id,
database_id=database_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    account_id: str,
    database_id: str,
    *,
    client: AuthenticatedClient,

) -> Optional[Database]:
    """ 
    Args:
        account_id (str):
        database_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Database
     """


    return (await asyncio_detailed(
        account_id=account_id,
database_id=database_id,
client=client,

    )).parsed



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ApiError")


@_attrs_define
class ApiError:
    """ 
        Attributes:
            status (float):
            message (str):
            request_id (str):
            path (str):
            method (str):
            log_id (str):
            handler (str):
            timestamp (str):
            causes (Union[Unset, Any]):
     """

    status: float
    message: str
    request_id: str
    path: str
    method: str
    log_id: str
    handler: str
    timestamp: str
    causes: Union[Unset, Any] = UNSET


    def to_dict(self) -> Dict[str, Any]:
        status = self.status

        message = self.message

        request_id = self.request_id

        path = self.path

        method = self.method

        log_id = self.log_id

        handler = self.handler

        timestamp = self.timestamp

        causes = self.causes


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "status": status,
            "message": message,
            "requestId": request_id,
            "path": path,
            "method": method,
            "logId": log_id,
            "handler": handler,
            "timestamp": timestamp,
        })
        if causes is not UNSET:
            field_dict["causes"] = causes

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = d.pop("status")

        message = d.pop("message")

        request_id = d.pop("requestId")

        path = d.pop("path")

        method = d.pop("method")

        log_id = d.pop("logId")

        handler = d.pop("handler")

        timestamp = d.pop("timestamp")

        causes = d.pop("causes", UNSET)

        api_error = cls(
            status=status,
            message=message,
            request_id=request_id,
            path=path,
            method=method,
            log_id=log_id,
            handler=handler,
            timestamp=timestamp,
            causes=causes,
        )

        return api_error




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ConnectionIPs")


@_attrs_define
class ConnectionIPs:
    """ 
        Attributes:
            private (List[str]):
            public (List[str]):
     """

    private: List[str]
    public: List[str]


    def to_dict(self) -> Dict[str, Any]:
        private = self.private





        public = self.public






        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "private": private,
            "public": public,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        private = cast(List[str], d.pop("private"))


        public = cast(List[str], d.pop("public"))


        connection_i_ps = cls(
            private=private,
            public=public,
        )

        return connection_i_ps




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="UsageCluster")


@_attrs_define
class UsageCluster:
    """ 
        Attributes:
            id (str):
            size (str):
            name (str):
            compute (Union[Unset, float]):
            out_same_region (Union[Unset, float]):
            out_different_region (Union[Unset, float]):
            out_internet (Union[Unset, float]):
     """

    id: str
    size: str
    name: str
    compute: Union[Unset, float] = UNSET
    out_same_region: Union[Unset, float] = UNSET
    out_different_region: Union[Unset, float] = UNSET
    out_internet: Union[Unset, float] = UNSET


    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        size = self.size

        name = self.name

        compute = self.compute

        out_same_region = self.out_same_region

        out_different_region = self.out_different_region

        out_internet = self.out_internet


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "id": id,
            "size": size,
            "name": name,
        })
        if compute is not UNSET:
            field_dict["compute"] = compute
        if out_same_region is not UNSET:
            field_dict["outSameRegion"] = out_same_region
        if out_different_region is not UNSET:
            field_dict["outDifferentRegion"] = out_different_region
        if out_internet is not UNSET:
            field_dict["outInternet"] = out_internet

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        size = d.pop("size")

        name = d.pop("name")

        compute = d.pop("compute", UNSET)

        out_same_region = d.pop("outSameRegion", UNSET)

        out_different_region = d.pop("outDifferentRegion", UNSET)

        out_internet = d.pop("outInternet", UNSET)

        usage_cluster = cls(
            id=id,
            size=size,
            name=name,
            compute=compute,
            out_same_region=out_same_region,
            out_different_region=out_different_region,
            out_internet=out_internet,
        )

        return usage_cluster




from __future__ import annotations
import getpass
import logging
import time

from typing import Iterable, List, Optional, Any
from contextlib import contextmanager
import datetime as dt
from datetime import datetime, timedelta

from tenacity import retry, TryAgain
from tenacity.wait import wait_fixed
from tenacity.stop import stop_after_delay

from exasol.saas.client import (
    openapi,
    Limits,
)
from exasol.saas.client.openapi.models.status import Status
from exasol.saas.client.openapi.api.databases import (
    create_database,
    delete_database,
    list_databases,
    get_database,
)
from exasol.saas.client.openapi.api.clusters import (
    get_cluster_connection,
    list_clusters,
)
from exasol.saas.client.openapi.api.security import (
    list_allowed_i_ps,
    add_allowed_ip,
    delete_allowed_ip,
)
from exasol.saas.client.openapi.types import UNSET


LOG = logging.getLogger(__name__)
LOG.setLevel(logging.INFO)


def timestamp_name(project_short_tag: str | None = None) -> str:
    """
    project_short_tag: Abbreviation of your project
    """
    timestamp = f'{datetime.now().timestamp():.0f}'
    owner = getpass.getuser()
    candidate = f"{timestamp}{project_short_tag or ''}-{owner}"
    return candidate[:Limits.MAX_DATABASE_NAME_LENGTH]


def wait_for_delete_clearance(start: dt.datetime):
    lifetime = datetime.now() - start
    if lifetime < Limits.MIN_DATABASE_LIFETIME:
        wait = Limits.MIN_DATABASE_LIFETIME - lifetime
        LOG.info(f"Waiting {int(wait.seconds)} seconds"
                 " before deleting the database.")
        time.sleep(wait.seconds)


class DatabaseStartupFailure(Exception):
    """
    If a SaaS database instance during startup reports a status other than
    successful.
    """


def create_saas_client(
        host: str,
        pat: str,
        raise_on_unexpected_status: bool = True,
) -> openapi.AuthenticatedClient:
    return openapi.AuthenticatedClient(
        base_url=host,
        token=pat,
        raise_on_unexpected_status = raise_on_unexpected_status,
    )


def _get_database_id(
        account_id: str,
        client: openapi.AuthenticatedClient,
        database_name: str,
) -> str:
    """
    Finds the database id, given the database name.
    """
    dbs = list_databases.sync(account_id, client=client)
    dbs = list(filter(lambda db: (db.name == database_name) and         # type: ignore
                                 (db.deleted_at is UNSET) and           # type: ignore
                                 (db.deleted_by is UNSET), dbs))        # type: ignore
    if not dbs:
        raise RuntimeError(f'SaaS database {database_name} was not found.')
    return dbs[0].id


def get_database_id(
        host: str,
        account_id: str,
        pat: str,
        database_name: str,
) -> str:
    """
    Finds the database id, given the database name.

    Args:
        host:           SaaS service URL.
        account_id:     User account ID
        pat:            Personal Access Token.
        database_name:  Database name.
    """
    with create_saas_client(host, pat) as client:
        return _get_database_id(account_id, client, database_name)


def get_connection_params(
        host: str,
        account_id: str,
        pat: str,
        database_id: str | None = None,
        database_name: str | None = None,
) -> dict[str, Any]:
    """
    Gets the database connection parameters, such as those required by pyexasol:
    - dns
    - user
    - password.
    Returns the parameters in a dictionary that can be used as kwargs when
    creating a connection, like in the code below:

    connection_params = get_connection_params(...)
    connection = pyexasol.connect(**connection_params)

    Args:
        host:           SaaS service URL.
        account_id:     User account ID
        pat:            Personal Access Token.
        database_id:    Database ID, id known.
        database_name:  Database name, in case the id is unknown.
    """

    with create_saas_client(host, pat) as client:
        if not database_id:
            if not database_name:
                raise ValueError(('To get SaaS connection parameters, '
                                  'either database name or database id must be provided.'))
            database_id = _get_database_id(account_id, client, database_name=database_name)
        clusters = list_clusters.sync(account_id,
                                      database_id,
                                      client=client)
        cluster_id = next(filter(lambda cl: cl.main_cluster, clusters)).id  # type: ignore
        connections = get_cluster_connection.sync(account_id,
                                                  database_id,
                                                  cluster_id,
                                                  client=client)
        if connections is None:
            raise RuntimeError('Failed to get the SaaS connection data.')
        connection_params = {
            'dsn': f'{connections.dns}:{connections.port}',
            'user': connections.db_username,
            'password': pat
        }
        return connection_params


class OpenApiAccess:
    """
    This class is meant to be used only in the context of the API
    generator repository while integration tests in other repositories are
    planned to only use fixture ``saas_database_id()``.
    """

    def __init__(self, client: openapi.AuthenticatedClient, account_id: str):
        self._client = client
        self._account_id = account_id

    def create_database(
            self,
            name: str,
            cluster_size: str = "XS",
            region: str = "eu-central-1",
            idle_time: timedelta | None = None
    ) -> Optional[openapi.models.Database]:
        def minutes(x: timedelta) -> int:
            return x.seconds // 60

        idle_time = idle_time or Limits.AUTOSTOP_MIN_IDLE_TIME
        cluster_spec = openapi.models.CreateDatabaseInitialCluster(
            name="my-cluster",
            size=cluster_size,
            auto_stop=openapi.models.AutoStop(
                enabled=True,
                idle_time=minutes(idle_time),
            ),
        )
        LOG.info(f"Creating database {name}")
        return create_database.sync(
            self._account_id,
            client=self._client,
            body=openapi.models.CreateDatabase(
                name=name,
                initial_cluster=cluster_spec,
                provider="aws",
                region=region,
            )
        )

    @contextmanager
    def _ignore_failures(self, ignore: bool = False):
        before = self._client.raise_on_unexpected_status
        self._client.raise_on_unexpected_status = not ignore
        yield self._client
        self._client.raise_on_unexpected_status = before

    def delete_database(self, database_id: str, ignore_failures=False):
        with self._ignore_failures(ignore_failures) as client:
            return delete_database.sync_detailed(
                self._account_id, database_id, client=client)

    def list_database_ids(self) -> Iterable[str]:
        dbs = list_databases.sync(self._account_id, client=self._client) or []
        return (db.id for db in dbs)

    @contextmanager
    def database(
            self,
            name: str,
            keep: bool = False,
            ignore_delete_failure: bool = False,
            idle_time: timedelta | None = None
    ):
        db = None
        start = datetime.now()
        try:
            db = self.create_database(name, idle_time=idle_time)
            yield db
            wait_for_delete_clearance(start)
        finally:
            db_repr = f"{db.name} with ID {db.id}" if db else None
            if db and not keep:
                LOG.info(f"Deleting database {db_repr}")
                response = self.delete_database(db.id, ignore_delete_failure)
                if response.status_code == 200:
                    LOG.info(f"Successfully deleted database {db_repr}.")
                else:
                    LOG.warning(f"Ignoring status code {response.status_code}.")
            elif not db:
                LOG.warning("Cannot delete db None")
            else:
                LOG.info(f"Keeping database {db_repr} as keep = {keep}")

    def get_database(
            self,
            database_id: str,
    ) -> Optional[openapi.models.database.Database]:
        return get_database.sync(
            self._account_id,
            database_id,
            client=self._client,
        )

    def wait_until_running(
            self,
            database_id: str,
            timeout: timedelta = timedelta(minutes=30),
            interval: timedelta = timedelta(minutes=2),
    ):
        success = [
            Status.RUNNING,
        ]

        @retry(wait=wait_fixed(interval), stop=stop_after_delay(timeout))
        def poll_status():
            db = self.get_database(database_id)
            if db.status not in success:
                print(f'status = {db.status}')
                raise TryAgain
            return db.status

        if poll_status() not in success:
            raise DatabaseStartupFailure()

    def clusters(
            self,
            database_id: str,
    ) -> Optional[List[openapi.models.Cluster]]:
        return list_clusters.sync(
            self._account_id,
            database_id,
            client=self._client,
        )

    def get_connection(
            self,
            database_id: str,
            cluster_id: str,
    ) -> Optional[openapi.models.ClusterConnection]:
        return get_cluster_connection.sync(
            self._account_id,
            database_id,
            cluster_id,
            client=self._client,
        )

    def list_allowed_ip_ids(self) -> Iterable[str]:
        ips = list_allowed_i_ps.sync(
            self._account_id,
            client=self._client,
        ) or []
        return (x.id for x in ips)

    def add_allowed_ip(
            self,
            cidr_ip: str = "0.0.0.0/0",
    ) -> Optional[openapi.models.allowed_ip.AllowedIP]:
        """
        Suggested values for cidr_ip:
        * 185.17.207.78/32
        * 0.0.0.0/0 = all ipv4
        * ::/0 = all ipv6
        """
        rule = openapi.models.CreateAllowedIP(
            name=timestamp_name(),
            cidr_ip=cidr_ip,
        )
        return add_allowed_ip.sync(
            self._account_id,
            client=self._client,
            body=rule,
        )

    def delete_allowed_ip(self, id: str, ignore_failures=False):
        with self._ignore_failures(ignore_failures) as client:
            return delete_allowed_ip.sync_detailed(
                self._account_id, id, client=client)

    @contextmanager
    def allowed_ip(
            self,
            cidr_ip: str = "0.0.0.0/0",
            keep: bool = False,
            ignore_delete_failure: bool = False,
    ):
        ip = None
        try:
            ip = self.add_allowed_ip(cidr_ip)
            yield ip
        finally:
            if ip and not keep:
                self.delete_allowed_ip(ip.id, ignore_delete_failure)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="PatchUserDatabases")


@_attrs_define
class PatchUserDatabases:
    """ 
        Attributes:
            delete (List[str]):
            add (List[str]):
     """

    delete: List[str]
    add: List[str]


    def to_dict(self) -> Dict[str, Any]:
        delete = self.delete





        add = self.add






        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "delete": delete,
            "add": add,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        delete = cast(List[str], d.pop("delete"))


        add = cast(List[str], d.pop("add"))


        patch_user_databases = cls(
            delete=delete,
            add=add,
        )

        return patch_user_databases




from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="DatabaseClusters")


@_attrs_define
class DatabaseClusters:
    """ 
        Attributes:
            total (int):
            running (int):
     """

    total: int
    running: int


    def to_dict(self) -> Dict[str, Any]:
        total = self.total

        running = self.running


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "total": total,
            "running": running,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total = d.pop("total")

        running = d.pop("running")

        database_clusters = cls(
            total=total,
            running=running,
        )

        return database_clusters




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['exasol',
 'exasol.saas.client',
 'exasol.saas.client.openapi',
 'exasol.saas.client.openapi.api',
 'exasol.saas.client.openapi.api.clusters',
 'exasol.saas.client.openapi.api.databases',
 'exasol.saas.client.openapi.api.extensions',
 'exasol.saas.client.openapi.api.files',
 'exasol.saas.client.openapi.api.platform',
 'exasol.saas.client.openapi.api.profile',
 'exasol.saas.client.openapi.api.security',
 'exasol.saas.client.openapi.api.usage',
 'exasol.saas.client.openapi.api.users',
 'exasol.saas.client.openapi.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=21.3.0',
 'httpx>=0.20.0,<0.28.0',
 'ifaddr>=0.2.0,<0.3.0',
 'python-dateutil>=2.8.0,<3.0.0',
 'requests>=2.31.0,<3.0.0',
 'tenacity>=8.2.3,<9.0.0',
 'types-requests>=2.31.0.6,<3.0.0.0']

entry_points = \
{'console_scripts': ['tbx = exasol.toolbox.tools.tbx:CLI']}

setup_kwargs = {
    'name': 'exasol-saas-api',
    'version': '0.7.0',
    'description': 'API enabling Python applications connecting to Exasol database SaaS instances and using their SaaS services',
    'long_description': '# SaaS API for Python\n\nAPI enabling Python applications connecting to Exasol database SaaS instances and using their SaaS services.\n\nThe model layer of this API is generated from the OpenAPI specification in JSON format of the SaaS API https://cloud.exasol.com/openapi.json using [openapi-python-client](https://github.com/openapi-generators/openapi-python-client).\n\nA GitHub action will check each morning if the generated model layer is outdated.\n\nSee\n* [User Guide](doc/user_guide/user-guide.md)\n* [Developer Guide](doc/developer_guide/developer_guide.md)\n',
    'author': 'Christoph Kuhnke',
    'author_email': 'christoph.kuhnke@exasol.com',
    'maintainer': 'Christoph Kuhnke',
    'maintainer_email': 'christoph.kuhnke@exasol.com',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<4.0',
}


setup(**setup_kwargs)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ClusterSize")


@_attrs_define
class ClusterSize:
    """ 
        Attributes:
            size (str):
            price (float):
            vcpu (float):
            ram (float):
            is_default (bool):
            name (str):
     """

    size: str
    price: float
    vcpu: float
    ram: float
    is_default: bool
    name: str


    def to_dict(self) -> Dict[str, Any]:
        size = self.size

        price = self.price

        vcpu = self.vcpu

        ram = self.ram

        is_default = self.is_default

        name = self.name


        field_dict: Dict[str, Any] = {}
        field_dict.update({
            "size": size,
            "price": price,
            "vcpu": vcpu,
            "ram": ram,
            "isDefault": is_default,
            "name": name,
        })

        return field_dict



    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        size = d.pop("size")

        price = d.pop("price")

        vcpu = d.pop("vcpu")

        ram = d.pop("ram")

        is_default = d.pop("isDefault")

        name = d.pop("name")

        cluster_size = cls(
            size=size,
            price=price,
            vcpu=vcpu,
            ram=ram,
            is_default=is_default,
            name=name,
        )

        return cluster_size

